<?php

namespace Unific\Connector\Plugin;

use Unific\Connector\Helper\Settings;
use Magento\Customer\Api\CustomerRepositoryInterface as CustomerRepository;
use Magento\Customer\Api\Data\CustomerInterface;
use Magento\Newsletter\Model\Subscriber;

class SubscriberPlugin extends AbstractPlugin
{
    protected $customerDataHelper;
    protected $customerRepository;

    protected $request;
    protected $subscriberFactory;

    /**
     * @param \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig
     * @param \Unific\Connector\Helper\Hmac $hmacHelper
     * @param \Unific\Connector\Helper\Message\Queue $queueHelper
     * @param \Magento\Framework\App\ProductMetadataInterface $productMetadata
     * @param \Magento\Framework\Module\ModuleListInterface $moduleList
     * @param \Unific\Connector\Helper\Data\Customer $customerDataHelper
     * @param \Magento\Customer\Api\CustomerRepositoryInterface $customerRepository
     * @param \Magento\Framework\App\RequestInterface $request
     * @param \Magento\Newsletter\Model\SubscriberFactory $subscriberFactory
     */
    public function __construct(
        \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig,
        \Unific\Connector\Helper\Hmac $hmacHelper,
        \Unific\Connector\Helper\Message\Queue $queueHelper,
        \Magento\Framework\App\ProductMetadataInterface $productMetadata,
        \Magento\Framework\Module\ModuleListInterface $moduleList,
        \Unific\Connector\Helper\Data\Customer $customerDataHelper,
        \Magento\Customer\Api\CustomerRepositoryInterface $customerRepository,
        \Magento\Framework\App\RequestInterface $request,
        \Magento\Newsletter\Model\SubscriberFactory $subscriberFactory
    ) {
        parent::__construct(
            $scopeConfig,
            $hmacHelper,
            $queueHelper,
            $productMetadata,
            $moduleList
        );

        $this->customerDataHelper = $customerDataHelper;
        $this->customerRepository = $customerRepository;

        $this->request = $request;
        $this->subscriberFactory = $subscriberFactory;
    }

    /**
     *
     * @param Subscriber $subject
     * @return Subscriber
     * @SuppressWarnings(PHPMD.UnusedFormalParameter)
     */
    public function afterSave(Subscriber $subject)
    {
        if ($this->scopeConfig->getValue('unific/connector/enabled') == 1 && $subject->getCustomerId() > 0) {
            $webhookSubject = 'customer/update';

            $this->customerDataHelper->setCustomer($this->customerRepository->getById($subject->getCustomerId()));

            if (filter_var($subject->getEmail(), FILTER_VALIDATE_EMAIL)) {
                $this->customerDataHelper->setOptinNewsletter($this->request->getParam('is_subscribed', $subject->isSubscribed()));

                $this->processWebhook(
                    $this->customerDataHelper->getCustomerInfo(),
                    $this->scopeConfig->getValue('unific/webhook/customer_endpoint'),
                    Settings::PRIORITY_CUSTOMER,
                    $webhookSubject
                );
            }
        }

        return $subject;
    }
}
